/*
 *   #   # ####     #    #######
 *   ##  # #       # #      #    Network Exploit Automation Tool
 *   # # # ####   #####     #
 *   #  ## #     #     #    #   (c) 2000 Mixter <mixter@newyorkoffice.com>,
 *   #   # #### #       #   #     (c) 2XS Ltd. -- http://2xss.com
 *
 *      command - functions for script language commands
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */

#ifndef COMMAND_H
#define COMMAND_H
#define _GNU_SOURCE
#include "cmdsocket.h"		// put it here since it's not ANSI compliant
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/resource.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include "language.h"
#include "util.h"
#include "smod.h"
#ifndef _NEAT_L
#include "xdr.h"
#endif
#include "cmdsocket.h"
#pragma interface "command"

class command : private cmdsocket
{
  private:
   // true if argument matches current version banner
   int match (const char *);
   // true if argument doesn't match current version banner
   int mnot (const char *);
   // true if OS version is matched (or if OS cannot be determined)
   int osmatch (const char *);
   // true if execution of argument succeeded
   int exec (const char *);
   // true if tcp connection has been established
   int tcp (const char *);
   // true if udp connection has been established
   int udp (const char *);
   // true if procedure specific remote procedure call succeeded
   int rpc (const char *);
   // true if all of the data could be sent
   int snd (const char *);
   // true if ftp can be accessed and written to with specified login/pass
   int writable (const char *);
   // true if hosts permitted to make a socks5 connection
   int socks (const char *);
   // true if hosts permitted to make a wingate connection
   int wgate (const char *);
   // true if program and script could both communicate with the shell
   int shl (const char *);
   // true if child forked and could listen to port, waiting to send text
   int lstn (const char *); 
   // true if hostname and optional comment could be written to file
   int log (const char *);
   // true if data was received and the given string was found in it
   int expect (const char *);
   // always true, sleep a specified amount of time
   int nap (const char *);
   // pipe descriptor to executed process
   int cpipe (const char *);
   // true if all data could be sent
   int xfer (const char *);
   // remote procedure call
   int rpcsend (const char *);
  protected:
   // remote OS detection (currently an interface to nmap)
   int osscan (unsigned int, char *);
  public:
   unsigned int longip;
   char *host;
   char *banner;
   int apply (int, const char *);
   void sockinit (void);
   command (void);
   ~command (void);
};
#endif
