/*
 *   #   # ####     #    #######
 *   ##  # #       # #      #    Network Exploit Automation Tool
 *   # # # ####   #####     #
 *   #  ## #     #     #    #   (c) 2000 Mixter <mixter@newyorkoffice.com>,
 *   #   # #### #       #   #     (c) 2XS Ltd. -- http://2xss.com
 *
 *      neat - main function and shell user interface
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */

#include "neat.h"

target *t;
smod *scanmode;
char *rulefiles[256]; // we can include up to 256 rulebase files
int nrules = 0;

void
usage(void)
{
  printf("\tUsage: neat [options]\n");
  printf("\t     -s\t\t\tenter interactive shell mode\n");
  printf("\t     -d path\t\tchange working directory (to find data files)\n");
  printf("\t     -l file\t\tredirect log messages to a file (default stdout)\n");
  printf("\t     -r file\t\tread rule base from a file (default %s)\n", DefaultRuleBase);
  printf("\t     -f file\t\tread scan logs from a file (default stdin)\n");
  printf("\t     -p command\t\tread scan logs from an input pipe\n");
  printf("\t     -h local address\tuse a local virtual ip address\n");
  printf("\t     -w n\t\tdelay each command for n milliseconds\n");
  printf("\t     -v \t\tverbose reporting and output\n");

  exit(0);
}

void
read_rules(char *rp)
{
 FILE *fp=fopen(rp, "r");
 char buffer[256];
 
 if(!fp)
  {
   fprintf(stderr, "FATAL: Can't read config file '%s'!\n", rp);
   exit(0);
  }
 
 while(fgets(buffer, 255, fp) != NULL)
  {
   strchop(buffer);
   if(buffer[0] == '#')
	   continue;
   rulefiles[nrules] = nstrdup(buffer);
   nrules++;
  }
}

int
main(int argc, char **argv)
{
  FILE *inp = stdin;
  scanmode = new smod;
  char *dirp = NULL;
  char *rulep;
  int shell = 0, opt = 0, delay = 0;

  rulep = nstrdup(DefaultRuleBase);

  printf("\n     OOH NEAT - Object Oriented Hierarchical Network Exploit Automation Tool\n");
  printf("     Version %s\t     (c) 2000 by Mixter, 2xs ltd.\n\n", VERSION);

  while ((opt = getopt(argc, argv, "sd:l:r:f:p:h:w:v")) != EOF)
    switch (opt)
      {
      case 'v':
        scanmode->verbose++;
        break;
      case 's':
	shell++;
	break;
      case 'd':
	dirp = optarg;
	break;
      case 'l':
	if ((scanmode->out = fopen(optarg, "a")) == NULL)
	  {
	    fprintf(stderr, LFileOpen, optarg);
	    usage();
	  }
	break;
      case 'r':
	delete rulep;
	rulep = nstrdup(optarg);
	break;
      case 'f':
	if ((inp = fopen(optarg, "r")) == NULL)
	  {
	    fprintf(stderr, LFileOpen, optarg);
	    usage();
	  }
	break;
      case 'p':
	if ((inp = popen(optarg, "r")) == NULL)
	  {
	    fprintf(stderr, LInvalidArg, optarg);
	    usage();
	  }
      case 'h':
	scanmode->myip.s_addr = resolve(optarg);
	break;
      case 'w':
	delay = atoi(optarg);
      default:
	usage();
      }

  if (!scanmode->myip.s_addr)
    scanmode->myip.s_addr = inet_gethostaddr();

  if (scanmode->myip.s_addr)
    fprintf(scanmode->out, "    [*] local ip address is %s\n", inet_ntoa(scanmode->myip));

  if (dirp != NULL)
    if (chdir(dirp))
      {
	fprintf(stderr, LChdir, optarg);
	usage();
      }

  read_rules(rulep);
  delete rulep;
  
  if (!shell)
    {
      if (inp == stdin)
	{
	  fprintf(scanmode->out, "    [*] entering interactive / standard input mode\n\n");
	  signal(SIGINT, sig_interrupt);
	  scanmode->mod_stdin = 1;
	}

      t = new target(inp);

      while (t->next())		// Next target host
	{
         if(scanmode->verbose)
	  fprintf(scanmode->out, "    [*] Next host %s...\n", inet_ntoa(t->GetIp()));

	  if (!intflag)
	    t->ApplyRuleset(rulefiles, nrules, delay);
#ifdef DEBUG2
	  else
	    fprintf(scanmode->out, "Debug: intflag 1 (neat.cc/main)\n");
#endif
	  if (delay)
	    usleep(delay);
	  intflag = 0;
	}
    }
  else
    {
      char buf[DefaultBufSize];
      command *c = new command;
      unsigned int cmds = 0;

      signal(SIGINT, sig_interrupt);
      scanmode->mod_stdin = 1;
      memset(buf, 0, DefaultBufSize);

      fprintf(scanmode->out, "    [*] entering shell mode\n\n[#0]NEAT> ");

      struct cmd_t
      {
	int id;
	char *name;
      };

      struct cmd_t com[] = {
	{C_MATCH, AMatch},
	{C_NOT, ANot},
	{C_EXEC, AExec},
	{C_TCP, ATcp},
	{C_UDP, AUdp},
	{C_RPC, ARpc},
	{C_SEND, ASend},
	{C_WRITABLE, AWritable},
	{C_SOCKS, ASocks},
	{C_WGATE, AWgate},
	{C_SHELL, AShell},
	{C_LISTEN, AListen},
	{C_XFER, AXfer},
	{C_LOG, ALog},
	{C_EXPECT, AExpect},
	{C_NAP, ANap},
	{C_RPCSEND, ARpcSend},
	{C_SLIST, ASockslist},
	{C_WLIST, AGatelist},
	{C_MODE, AMode},
	{C_TIMEOUT, ATimeout},
	{C_OSSCAN, AOSScan},
	{C_OSMATCH, AOSMatch},
	{C_FROM, AFrom},
	{C_INCLUDE, AInclude},
	{0, ""},
      };

      while (fgets(buf, DefaultBufSize, inp) != NULL)
	{
	  int i = 0;
	  char *arg;

	  if (!strncasecmp(buf, AExit, strlen(AExit)))
	    {
	      fprintf(scanmode->out, "    [*] bye\n");
	      return 0;
	    }
	  else if (!strncasecmp(buf, ARun, strlen(ARun)))
	    {
	      char arg0[DefaultBufSize];
	      char arg1[DefaultBufSize];

	      if (strscpy(buf, arg0, arg1, ' ') == NULL)
		{
		  fprintf(stderr, LArgMissing, argv[0], ARun);
		  continue;
		}
	      system(strchop(arg1));
	      goto ok;
	    }
	  else if (!strncasecmp(buf, ABanner, strlen(ABanner)))
	    {
	      char arg0[DefaultBufSize];
	      char arg1[DefaultBufSize];

	      if (strscpy(buf, arg0, arg1, ' ') == NULL)
		{
		  fprintf(stderr, LArgMissing, argv[0], ABanner);
		  continue;
		}
	      strncpy(c->banner, strchop(arg1), DefaultBufSize);
	      fprintf(scanmode->out, "    [banner: %s]\n", c->banner);
	      goto ok;
	    }

	  arg = strtok(buf, " ");
	  if (!arg)
	    goto notok;

	  if (!strncasecmp(arg, AHost, strlen(AHost)))
	    {
	      arg = strtok(NULL, " ");
	      if (!(arg) || !(c->longip = resolve(strchop(arg))))
		{
		  fprintf(stderr, LArgMissing, argv[0], AHost);
		  continue;
		}
	      strncpy(c->host, arg, DefaultBufSize);
	      fprintf(scanmode->out, "    [host: %s]\n", c->host);
	    }
	  else
	    while (com[i].id && com[i].name)
	      {
		if (!strncasecmp(arg, com[i].name, strlen(com[i].name)))
		  {
		    arg = strtok(NULL, " ");
		    if (!arg)
		      {
			fprintf(stderr, LArgMissing, argv[0], com[i].name);
			continue;
		      }

		    cmds++;
		    (void) strchop(arg);

		    if (com[i].id < 0xc8)
		      {
			if (!c->apply(com[i].id, arg))
			  fprintf(scanmode->out,
				  "[command returned failure]\n");
		      }
		    else
		      switch (com[i].id)
			{
			case C_SLIST:
			  scanmode->slist(arg);
			  fprintf(scanmode->out, "[new socks list: %s]\n",
				  arg);
			  break;
			case C_WLIST:
			  scanmode->wlist(arg);
			  fprintf(scanmode->out, "[new wingate list: %s]\n",
				  arg);
			  break;
			case C_MODE:
			  scanmode->parse(arg);
			  fprintf(scanmode->out, "[setting mode: %s]\n", arg);
			  break;
			case C_TIMEOUT:
			  scanmode->timeout = atoi(arg);
			  fprintf(scanmode->out,
				  "[new timeout: %d seconds]\n",
				  scanmode->timeout);
			  break;
			default:
			  goto notok;
			}
		    goto ok;
		  }
		i++;
	      }
	notok:
	  fprintf(scanmode->out, "     [*] NEAT shell commands summary\n\n");
	  fprintf(scanmode->out,
		  "  host <address>                        banner <string>\n");
	  fprintf(scanmode->out,
		  "  exit                                  run <system command>\n");
	  fprintf(scanmode->out,
		  "  mode <stop/interactive/bounce/fast>   timeout <seconds>\n");
	  fprintf(scanmode->out,
		  "  slist <file>                          wlist <file>\n");
	  fprintf(scanmode->out,
		  "  match <banner>                        shell <file or text>\n");
	  fprintf(scanmode->out,
		  "  socks <file>                          wgate <file>\n");
	  fprintf(scanmode->out,
		  "  tcp <port>                            udp <port>\n");
	  fprintf(scanmode->out,
		  "  exec <program>                        send <file, text or variable>\n");
	  fprintf(scanmode->out,
		  "  log <file[/comment]>                  nap <sec[/usec]>\n");
	  fprintf(scanmode->out,
		  "  exp <pattern[/tries]>                 writable <login/pass>\n");
	  fprintf(scanmode->out,
		  "  rpc <prog/ver/proto>                  rpcsend <callnr/file or text>\n");
	  fprintf(scanmode->out,
		  "  listen <port/file>                    xfer <local>remote>\n");
	  fprintf(scanmode->out,
		  " Note: You need to set a host before executing any connection-related command.\n");
	ok:
	  if ((strlen(c->host)) && (strlen(c->banner)))
	    fprintf(scanmode->out, "[t:%s b:%s #%d]NEAT> ", c->host,
		    c->banner, cmds);
	  else if ((strlen(c->host)))
	    fprintf(scanmode->out, "[t:%s #%d]NEAT> ", c->host, cmds);
	  else if ((strlen(c->banner)))
	    fprintf(scanmode->out, "[b:%s #%d]NEAT> ", c->banner, cmds);
	  else
	    fprintf(scanmode->out, "[#%d]NEAT> ", cmds);
	  fflush(scanmode->out);
	}

    }

  printf("\n     NEAT Finished. Have a NEAT day!\n");

  delete scanmode;

  return 0;
}
